@@ -14,6 +14,7 @@
  *	31-Jul-2002 Lineo Japan, Inc.  for ARCH_PXA
  *	12-Dec-2002 Sharp Corporation for Poodle and Corgi
  *	27-Feb-2003 Sharp modified for Poodle
+ *	30-Jul-2004 Lineo Solutions, Inc. for Spitz
  *
  */
 
@@ -143,6 +144,7 @@ unsigned long ads7846_rw(ulong data)
 #endif
 }
 
+
 #if defined(CONFIG_ARCH_PXA_CORGI)
 
 //#define WAIT_AFTER_SYNC_HS 151 /* 41us */
@@ -151,9 +153,21 @@ static unsigned long wait_after_sync_hs = 0;
 #define WAIT_AFTER_SYNC_HS wait_after_sync_hs
 #define SyncHS()	while((GPLR(GPIO_HSYNC) & GPIO_bit(GPIO_HSYNC)) == 0);\
 while((GPLR(GPIO_HSYNC) & GPIO_bit(GPIO_HSYNC)) != 0);
+#if defined(CONFIG_CPU_PXA27X)
+#define CCNT(a)    { a = pxa27x_get_ccnt(); }
+#define CCNT_ON()  { pxa27x_enable_ccnt(); }
+#define CCNT_OFF() { pxa27x_disable_ccnt(); }
+#else
 #define CCNT(a)		asm("mrc p14, 0, %0, C1, C0, 0" : "=r"(a))
 #define CCNT_ON()	{int pmnc = 1;asm("mcr p14, 0, %0, C0, C0, 0" : : "r"(pmnc));}
 #define CCNT_OFF()	{int pmnc = 0;asm("mcr p14, 0, %0, C0, C0, 0" : : "r"(pmnc));}
+#endif
+#if defined(CONFIG_FB_SHARPSL_PXA)
+#define WAIT_HS_333_VGA		11582	// 17.995us
+#define WAIT_HS_400_VGA		6250	// 15.024us
+#define WAIT_HS_333_QVGA	27938	// 42.100us
+#define WAIT_HS_400_QVGA	15699	// 37.738us
+#else
 #if 1
 #define WAIT_HS_333_VGA		5791	// 17.995us
 #define WAIT_HS_400_VGA		7013	// 17.615us
@@ -165,20 +179,40 @@ while((GPLR(GPIO_HSYNC) & GPIO_bit(GPIO_HSYNC)) != 0);
 #define WAIT_HS_333_QVGA	13935	// 42.0us
 #define WAIT_HS_400_QVGA	16720	// 42.0us
 #endif
+#endif
 #define LCD_MODE_480 0
 #define LCD_MODE_320 1
 #define LCD_MODE_240 2
+#if defined(CONFIG_FB_SHARPSL_PXA)
+extern int sharpsl_pxafb_lcdMode;
+extern int sharpsl_pxafb_isblank;
+#define	FB_LCDMODE	sharpsl_pxafb_lcdMode
+#define	FB_ISBLANK	sharpsl_pxafb_isblank
+#elif defined(CONFIG_FB_CORGI)
 extern int w100fb_lcdMode;
 extern int w100fb_isblank;
+#define	FB_LCDMODE	w100fb_lcdMode
+#define	FB_ISBLANK	w100fb_isblank
+#endif
 extern unsigned long cccr_reg;
 
+static unsigned long ccnt(void)
+{
+	unsigned long timer;
+
+	CCNT(timer);
+	return timer;
+}
+
 static int sync_receive_data_send_cmd(int doRecive,int doSend,int sCmd)
 {
 	int pos = 0;
 	unsigned long timer1,timer2;
 	unsigned long wait_time;
 
-	if (w100fb_isblank) {
+#if defined(CONFIG_FB_CORGI) || defined(CONFIG_FB_SHARPSL_PXA)
+	if (FB_ISBLANK)
+	{
 		/* read SSDR */
 		if(doRecive){
 			pos = ssp_only_get_dac_val(CS_ADS7846);
@@ -190,22 +224,29 @@ static int sync_receive_data_send_cmd(int doRecive,int doSend,int sCmd)
 		}
 		return pos;
 	}
+#endif
 
 	if (cccr_reg == 0x145) {
 		// 332MHz
-		if (w100fb_lcdMode == LCD_MODE_480) {
+#if defined(CONFIG_FB_CORGI) || defined(CONFIG_FB_SHARPSL_PXA)
+		if (FB_LCDMODE == LCD_MODE_480) {
 			wait_time = WAIT_HS_333_VGA - wait_after_sync_hs;
 		}
-		else {
+		else
+#endif
+		{
 			wait_time = WAIT_HS_333_QVGA - wait_after_sync_hs;
 		}
 	}
 	else {
 		// 400MHz
-		if (w100fb_lcdMode == LCD_MODE_480) {
+#if defined(CONFIG_FB_CORGI) || defined(CONFIG_FB_SHARPSL_PXA)
+		if (FB_LCDMODE == LCD_MODE_480) {
 			wait_time = WAIT_HS_400_VGA - wait_after_sync_hs;
 		}
-		else {
+		else
+#endif
+		{
 			wait_time = WAIT_HS_400_QVGA - wait_after_sync_hs;
 		}
 	}
@@ -229,13 +270,17 @@ static int sync_receive_data_send_cmd(int doRecive,int doSend,int sCmd)
 
 		/* Wait after HSync */
 		CCNT(timer2);
-		if (w100fb_lcdMode != LCD_MODE_480) {
+#if defined(CONFIG_FB_CORGI) || defined(CONFIG_FB_SHARPSL_PXA)
+		if (FB_LCDMODE != LCD_MODE_480) {
 			while((timer2-timer1) < wait_time){
-				CCNT(timer2);
+//				CCNT(timer2);
+				timer2 = ccnt();
 			}
 			ssp_put_dac_val(sCmd, CS_ADS7846);
 		}
-		else {
+		else
+#endif
+		{
 			/* timeout */
 			SyncHS();
 			/* get OSCR */
@@ -243,7 +288,8 @@ static int sync_receive_data_send_cmd(int doRecive,int doSend,int sCmd)
 			/* Wait after HSync */
 			CCNT(timer2);
 			while((timer2-timer1) < wait_time){
-				CCNT(timer2);
+//				CCNT(timer2);
+				timer2 = ccnt();
 			}
 			ssp_put_dac_val(sCmd, CS_ADS7846);
 		}
@@ -906,6 +952,9 @@ static void ts_interrupt_main(int irq, int isTimer)
 #endif
 			tc.pressure = 500;
 #elif defined(CONFIG_ARCH_PXA_CORGI)
+#if defined(CONFIG_ARCH_PXA_SPITZ)
+		lock_FCS(POWER_MODE_TOUCH, 1);	// not enter FCS mode.
+#endif
 		if( pos_dt.xd && pos_dt.yd && Pressure ) {
 			tc.pressure = 500;
 #else
@@ -939,7 +988,7 @@ static void ts_interrupt_main(int irq, int isTimer)
 			tc.pressure = 0;
 			new_data();
 		}
-#if defined(CONFIG_ARCH_PXA_POODLE)
+#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_SPITZ)
 		lock_FCS(POWER_MODE_TOUCH, 0);	// enter FCS mode.
 #endif
 		/* Clear detect */
