@@ -102,7 +102,20 @@
  *
  *  09-04-2002  tglx: fixed write_verify (John Hall (john.hall@optionexist.co.uk))
  *
- * $Id: nand.c,v 1.31 2002/09/04 11:19:33 gleixner Exp $
+ *  11-11-2002  tglx: fixed debug output in nand_write_page 
+ *		(John Hall (john.hall@optionexist.co.uk))
+ *
+ *  11-25-2002  tglx: Moved device ID/ manufacturer ID from nand_ids.h
+ *		Splitted device ID and manufacturer ID table. 
+ *		Removed CONFIG_MTD_NAND_ECC, as it defaults to ECC_NONE for
+ *		mtd->read / mtd->write and is controllable by the fs driver
+ *		for mtd->read_ecc / mtd->write_ecc
+ *		some minor cleanups
+ *
+ *  12-05-2000  tglx: Dave Ellis (DGE@sixnetio) provided the fix for
+ *		WRITE_VERIFY long time ago. Thanks for remembering me.	
+ *	
+ * $Id: nand.c,v 1.36 2002/12/05 20:59:11 gleixner Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -122,6 +135,7 @@
  *     18-Sep-2002 Lineo Japan, Inc.  add dev_ready() call after read
  *     17-Sep-2002 Lineo Japan, Inc.  add code for post-badblock
  *     14-Mar-2003 Sharp wait for ready in read_oob()
+ *     09-Sep-2003 SHARP              support TC6393XB controller for Tosa
  */
 
 #include <linux/delay.h>
@@ -130,7 +144,6 @@
 #include <linux/types.h>
 #include <linux/mtd/mtd.h>
 #include <linux/mtd/nand.h>
-#include <linux/mtd/nand_ids.h>
 #include <linux/mtd/nand_ecc.h>
 #include <linux/interrupt.h>
 #include <asm/io.h>
@@ -364,13 +377,17 @@ static int nand_wait(struct mtd_info *mtd, struct nand_chip *this, int state)
  *	Nand_page_program function is used for write and writev !
  */
 static int nand_write_page (struct mtd_info *mtd, struct nand_chip *this,
-			    const int page, int col, int last, u_char *oob_buf, int oobsel)
+			    int page, int col, int last, u_char *oob_buf, int oobsel)
 {
 	int 	i, status;
 	u_char	ecc_code[6], *oob_data;
 	int	eccmode = oobsel ? this->eccmode : NAND_ECC_NONE;
 	int  	*oob_config = oobconfigs[oobsel];
 	int	ecc_ret = 0;
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+	unsigned short data_w;
+	u_char data_b;
+#endif
 
 	START_MEASUREMENT(nand_write_page);
 	COUNTER_INC(nand_write_nr_pages);
@@ -428,9 +445,16 @@ static int nand_write_page (struct mtd_info *mtd, struct nand_chip *this,
 	/* No ecc and software ecc 3/256, write all */
 	case NAND_ECC_NONE:
 	case NAND_ECC_SOFT:
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+		for (i = 0; i < mtd->oobblock; i+=2) {
+			writew ( (this->data_poi[i+1] << 8) | this->data_poi[i] , this->IO_ADDR_W);
+		}
+		break;
+#else
 		for (i = 0; i < mtd->oobblock; i++) 
 			writeb ( this->data_poi[i] , this->IO_ADDR_W);
 		break;
+#endif
 		
 	/* Hardware ecc 3 byte / 256 data, write first half, get ecc, then second, if 512 byte pagesize */	
 	case NAND_ECC_HW3_256:		
@@ -465,8 +489,14 @@ static int nand_write_page (struct mtd_info *mtd, struct nand_chip *this,
 	/* Hardware ecc 6 byte / 512 byte data, write full page */	
 	case NAND_ECC_HW6_512:	
 		this->enable_hwecc (NAND_ECC_WRITE);	/* enable hardware ecc logic */
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+		for (i = 0; i < mtd->oobblock; i+=2) {
+			writew ( (this->data_poi[i+1] << 8) | this->data_poi[i] , this->IO_ADDR_W);
+		}
+#else
 		for (i = 0; i < mtd->oobblock; i++) 
 			writeb ( this->data_poi[i] , this->IO_ADDR_W);
+#endif
 		ecc_ret = this->calculate_ecc (NULL, &(ecc_code[0]));
 		for (i = 0; i < 6; i++)
 			oob_data[oob_config[i]] = ecc_code[i];
@@ -484,8 +514,15 @@ static int nand_write_page (struct mtd_info *mtd, struct nand_chip *this,
 	}
 	
 	/* Write out OOB data */
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+
+	for (i = 0; i <  mtd->oobsize; i+=2) {
+		writew ( (oob_data[i+1] << 8) | oob_data[i] , this->IO_ADDR_W);
+	}
+#else
 	for (i = 0; i <  mtd->oobsize; i++)
 		writeb ( oob_data[i] , this->IO_ADDR_W);
+#endif
 
 	/* Send command to actually program the data */
 	this->cmdfunc (mtd, NAND_CMD_PAGEPROG, -1, -1);
@@ -495,7 +532,7 @@ static int nand_write_page (struct mtd_info *mtd, struct nand_chip *this,
 
 	/* See if device thinks it succeeded */
 	if (status & 0x01) {
-		DEBUG (MTD_DEBUG_LEVEL0, "nand_write_ecc: " "Failed write, page 0x%08x, ", page);
+		DEBUG (MTD_DEBUG_LEVEL0, "%s: " "Failed write, page 0x%08x, ", __func__, page);
 		ACCUMULATE_ELAPSED_TIME(nand_write_page);
 		return -EIO;
 	}
@@ -526,23 +563,55 @@ static int nand_write_page (struct mtd_info *mtd, struct nand_chip *this,
 	}
 	/* Loop through and verify the data */
 	for (i = col; i < last; i++) {
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+		if((i-col) & 1){
+			data_b = data_w >> 8;
+		}else{
+			data_w = readw (this->IO_ADDR_W);
+			data_b = data_w & 0xff;
+				
+		}
+		if (this->data_poi[i] != data_b) {
+			printk (KERN_WARNING "%s: Failed write verify, %d at page 0x%08x, col=%d last=%d\n", __func__, i, page, col, last);
+			ACCUMULATE_ELAPSED_TIME(nand_write_verify);
+			ACCUMULATE_ELAPSED_TIME(nand_write_page);
+			return -EIO;
+		}
+#else
 		if (this->data_poi[i] != readb (this->IO_ADDR_R)) {
 			printk (KERN_WARNING "%s: Failed write verify, %d at page 0x%08x, col=%d last=%d\n", __func__, i, page, col, last);
 			ACCUMULATE_ELAPSED_TIME(nand_write_verify);
 			ACCUMULATE_ELAPSED_TIME(nand_write_page);
 			return -EIO;
 		}
+#endif
 	}
 
 	/* check, if we have a fs-supplied oob-buffer */
 	if (oob_buf) {
 		for (i = 0; i < mtd->oobsize; i++) {
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+			if(i & 1){
+				data_b = data_w >> 8;
+			}else{
+				data_w = readw (this->IO_ADDR_W);
+				data_b = data_w & 0xff;
+				
+			}
+			if (oob_data[i] != data_b) {
+				printk (KERN_WARNING "%s: Failed write verify, oob %d of page 0x%08x\n", __func__, i, page);
+				ACCUMULATE_ELAPSED_TIME(nand_write_verify);
+				ACCUMULATE_ELAPSED_TIME(nand_write_page);
+				return -EIO;
+			}
+#else
 			if (oob_data[i] != readb (this->IO_ADDR_R)) {
 				printk (KERN_WARNING "%s: Failed write verify, oob %d of page 0x%08x\n", __func__, i, page);
 				ACCUMULATE_ELAPSED_TIME(nand_write_verify);
 				ACCUMULATE_ELAPSED_TIME(nand_write_page);
 				return -EIO;
 			}
+#endif
 		}
 	} else {
 		if (eccmode != NAND_ECC_NONE && !col && last == mtd->oobblock) {
@@ -555,8 +624,19 @@ static int nand_write_page (struct mtd_info *mtd, struct nand_chip *this,
 			case NAND_ECC_HW6_512: ecc_bytes = 6; break;
 			}
 
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+			for (i = 0; i < mtd->oobsize; i++) {
+				if(i & 1){
+					oob_data[i] = data_w >> 8;
+				}else{
+					data_w = readw (this->IO_ADDR_R);
+					oob_data[i] = data_w;
+				}
+			}
+#else
 			for (i = 0; i < mtd->oobsize; i++)
 				oob_data[i] = readb (this->IO_ADDR_R);
+#endif
 
 			for (i = 0; i < ecc_bytes; i++) {
 				if (oob_data[oob_config[i]] != ecc_code[i]) {
@@ -826,7 +906,17 @@ static int nand_read_oob (struct mtd_info *mtd, loff_t from, size_t len, size_t
 		 * oob data, let the device transfer the data !
 		 */
 		for (i = 0; i < tmp_len; i++) {
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+			unsigned short data;
+			if(i & 1){
+				buf[i] = data >> 8;
+			}else{
+				data = readw (this->IO_ADDR_R);
+				buf[i] = data;
+			}
+#else
 			buf[i] = readb (this->IO_ADDR_R);
+#endif
 			if ((col++ & (mtd->oobsize - 1)) == (mtd->oobsize - 1))
 				udelay (this->chip_delay);
 		}
@@ -927,7 +1017,7 @@ static int nand_write_ecc (struct mtd_info *mtd, loff_t to, size_t len,
 			cnt = mtd->oobblock;
 		} else {
 			cnt = min(len - written, mtd->oobblock);
-			memcpy(&this->data_buf[col], &buf[written + col], cnt);
+			memcpy(&this->data_buf[col], &buf[written], cnt);
 			this->data_poi = this->data_buf;		
 		}
 		/* We use the same function for write and writev !) */
@@ -972,6 +1062,10 @@ static int nand_write_oob (struct mtd_info *mtd, loff_t to, size_t len, size_t *
 {
 	int i, column, page, status, ret = 0;
 	struct nand_chip *this = mtd->priv;
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+	unsigned short tmp_buf[32]; // mtd->oobsize
+	u_char *p_tmp_buf;
+#endif
 
 	DEBUG (MTD_DEBUG_LEVEL3, "nand_write_oob: to = 0x%08x, len = %i\n", (unsigned int) to, (int) len);
 
@@ -1010,6 +1104,17 @@ static int nand_write_oob (struct mtd_info *mtd, loff_t to, size_t len, size_t *
 
 	/* Write out desired data */
 	this->cmdfunc (mtd, NAND_CMD_SEQIN, mtd->oobblock, page);
+
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+	memset(tmp_buf,0xff,mtd->oobsize);
+	p_tmp_buf = (u_char *)tmp_buf;
+
+	for (i = 0; i < len; i++)
+		p_tmp_buf[i+column] = buf[i];
+
+	for (i = 0; i < mtd->oobsize; i++)
+		writew (tmp_buf[i], this->IO_ADDR_W);
+#else
 	/* prepad 0xff for partial programming */
 	for (i = 0; i < column; i++)
 		writeb (0xff, this->IO_ADDR_W);
@@ -1019,6 +1124,7 @@ static int nand_write_oob (struct mtd_info *mtd, loff_t to, size_t len, size_t *
 	/* postpad 0xff for partial programming */
 	for (i = len + column; i < mtd->oobsize; i++)
 		writeb (0xff, this->IO_ADDR_W);
+#endif
 
 	/* Send command to program the OOB data */
 	this->cmdfunc (mtd, NAND_CMD_PAGEPROG, -1, -1);
@@ -1040,11 +1146,29 @@ static int nand_write_oob (struct mtd_info *mtd, loff_t to, size_t len, size_t *
 
 	/* Loop through and verify the data */
 	for (i = 0; i < len; i++) {
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+		unsigned short data;
+		if(i & 1){
+			if (buf[i] != (data >> 8)){
+				DEBUG (MTD_DEBUG_LEVEL0, "nand_write_oob: " "Failed write verify, page 0x%08x\n", page);
+				ret = -EIO;
+				goto out;
+			}
+		}else{
+			data = readw (this->IO_ADDR_R);
+			if (buf[i] != (data & 0xff)){
+				DEBUG (MTD_DEBUG_LEVEL0, "nand_write_oob: " "Failed write verify, page 0x%08x\n", page);
+				ret = -EIO;
+				goto out;
+			}
+		}
+#else
 		if (buf[i] != readb (this->IO_ADDR_R)) {
 			DEBUG (MTD_DEBUG_LEVEL0, "nand_write_oob: " "Failed write verify, page 0x%08x\n", page);
 			ret = -EIO;
 			goto out;
 		}
+#endif
 	}
 #endif
 
@@ -1437,29 +1561,39 @@ int nand_scan (struct mtd_info *mtd)
 	this->cmdfunc (mtd, NAND_CMD_READID, 0x00, -1);
 
 	/* Read manufacturer and device IDs */
+#ifdef CONFIG_MTD_NAND_SHARP_SL_TC6393
+	nand_maf_id = readw (this->IO_ADDR_R);
+	nand_dev_id = nand_maf_id >> 8;
+	nand_maf_id = nand_maf_id & 0xff;
+#else
 	nand_maf_id = readb (this->IO_ADDR_R);
 	nand_dev_id = readb (this->IO_ADDR_R);
+#endif
 
 	/* Print and store flash device information */
 	for (i = 0; nand_flash_ids[i].name != NULL; i++) {
-		if (nand_maf_id == nand_flash_ids[i].manufacture_id && nand_dev_id == nand_flash_ids[i].model_id) {
-			if (!mtd->size) {
-				mtd->name = nand_flash_ids[i].name;
-				mtd->erasesize = nand_flash_ids[i].erasesize;
-				mtd->size = (1 << nand_flash_ids[i].chipshift);
-				mtd->eccsize = 256;
-				if (nand_flash_ids[i].page256) {
-					mtd->oobblock = 256;
-					mtd->oobsize = 8;
-					this->page_shift = 8;
-				} else {
-					mtd->oobblock = 512;
-					mtd->oobsize = 16;
-					this->page_shift = 9;
-				}
+		if (nand_dev_id == nand_flash_ids[i].id && !mtd->size) {
+			mtd->name = nand_flash_ids[i].name;
+			mtd->erasesize = nand_flash_ids[i].erasesize;
+			mtd->size = (1 << nand_flash_ids[i].chipshift);
+			mtd->eccsize = 256;
+			if (nand_flash_ids[i].page256) {
+				mtd->oobblock = 256;
+				mtd->oobsize = 8;
+				this->page_shift = 8;
+			} else {
+				mtd->oobblock = 512;
+				mtd->oobsize = 16;
+				this->page_shift = 9;
 			}
+			/* Try to identify manufacturer */
+			for (i = 0; nand_manuf_ids[i].id != 0x0; i++) {
+				if (nand_manuf_ids[i].id == nand_maf_id)
+					break;
+			}	
 			printk (KERN_INFO "NAND device: Manufacture ID:"
-				" 0x%02x, Chip ID: 0x%02x (%s)\n", nand_maf_id, nand_dev_id, mtd->name);
+				" 0x%02x, Chip ID: 0x%02x (%s %s)\n", nand_maf_id, nand_dev_id, 
+				nand_manuf_ids[i].name , mtd->name);
 			break;
 		}
 	}
@@ -1469,11 +1603,13 @@ int nand_scan (struct mtd_info *mtd)
 	 * if 3byte/512byte hardware ECC is selected and we have 256 byte pagesize
 	 * fallback to software ECC 
 	*/
-#ifdef CONFIG_MTD_NAND_ECC	
 	this->eccsize = 256;	/* set default eccsize */	
 
 	switch (this->eccmode) {
 
+#ifdef CONFIG_ARCH_SHARP_SL
+	case NAND_ECC_HW6_512:
+#endif
 	case NAND_ECC_HW3_512: 
 		if (mtd->oobblock == 256) {
 			printk (KERN_WARNING "512 byte HW ECC not possible on 256 Byte pagesize, fallback to SW ECC \n");
@@ -1503,9 +1639,6 @@ int nand_scan (struct mtd_info *mtd)
 		printk (KERN_WARNING "Invalid NAND_ECC_MODE %d\n", this->eccmode);
 		BUG();	
 	}	
-#else
-	this->eccmode = NAND_ECC_NONE;
-#endif
 
 	/* Initialize state, waitqueue and spinlock */
 	this->state = FL_READY;
