@@ -28,6 +28,8 @@
  * ChangLog:
  *     23-Oct-2002 SHARP  add functions for CONFIG_MTD_NAND_LOGICAL_ADDRESS_ACCESS
  *     14-Mar-2003 Sharp wait for ready
+ *     01-Jun-2004 Lineo Solutions, Inc.  for Spitz
+ *     26-Aug-2004 Sharp cancel write-protect at resume
  */
 
 #include <linux/slab.h>
@@ -97,10 +99,26 @@ static int oobconfigs[][6] = {
 /*
  * Define partitions for flash device
  */
-#define DEFAULT_NUM_PARTITIONS 3
 static struct mtd_info** sharp_sl_nand_part_mtdp;
 static int nr_partitions;
 static struct mtd_partition sharp_sl_nand_default_partition_info[] = {
+#if defined(CONFIG_ARCH_PXA_SPITZ)
+    {
+	.name = "smf",
+	.offset = 0,
+	.size = 7 * 1024 * 1024,
+    },
+    {
+	.name = "root",
+	.offset = 7 * 1024 * 1024,
+	.size = 53 * 1024 * 1024,
+    },
+    {
+	.name = "home",
+	.offset = 60 * 1024 * 1024,
+	.size = (128 - 60) * 1024 * 1024,
+    },
+#else
     {
 	.name = "NAND flash partition 0",
 	.offset = 0,
@@ -116,9 +134,12 @@ static struct mtd_partition sharp_sl_nand_default_partition_info[] = {
 	.offset = 37 * 1024 * 1024,
 	.size = (64 - 37) * 1024 * 1024,
     },
+#endif
 };
 static struct mtd_partition* sharp_sl_nand_partition_info;
 
+#define DEFAULT_NUM_PARTITIONS (sizeof(sharp_sl_nand_default_partition_info) / sizeof(struct mtd_partition))
+
 
 /* 
  *	hardware specific access to control-lines
@@ -860,6 +881,8 @@ static void
 sharp_sl_nand_resume(struct mtd_info* mtd)
 {
     printk("%s\n", __func__);
+
+    CPLD_REG(FLASHCTL) |= FLWP;
 }
 
 
