@@ -11,7 +11,7 @@
 
     Copyright (C) 1999 David A. Hinds -- dahinds@users.sourceforge.net
 
-    pcnet_cs.c 1.144 2001/11/07 04:06:56
+    pcnet_cs.c 1.149 2002/06/29 06:27:37
     
     The network driver code is based on Donald Becker's NE2000 code:
 
@@ -41,9 +41,11 @@
 #include <linux/string.h>
 #include <linux/timer.h>
 #include <linux/delay.h>
+#include <linux/ethtool.h>
 #include <asm/io.h>
 #include <asm/system.h>
 #include <asm/byteorder.h>
+#include <asm/uaccess.h>
 
 #include <linux/netdevice.h>
 #include <../drivers/net/8390.h>
@@ -56,10 +58,10 @@
 #include <pcmcia/ds.h>
 #include <pcmcia/cisreg.h>
 
-#define PCNET_CMD	0x00
-#define PCNET_DATAPORT	0x10	/* NatSemi-defined port window offset. */
-#define PCNET_RESET	0x1f	/* Issue a read to reset, a write to clear. */
-#define PCNET_MISC	0x18	/* For IBM CCAE and Socket EA cards */
+#define PCNET_CMD	(0x00<<PCMCIA_ADDRESS_SHIFT)
+#define PCNET_DATAPORT	(0x10<<PCMCIA_ADDRESS_SHIFT)	/* NatSemi-defined port window offset. */
+#define PCNET_RESET	(0x1f<<PCMCIA_ADDRESS_SHIFT)	/* Issue a read to reset, a write to clear. */
+#define PCNET_MISC	(0x18<<PCMCIA_ADDRESS_SHIFT)	/* For IBM CCAE and Socket EA cards */
 
 #define PCNET_START_PG	0x40	/* First page of TX buffer */
 #define PCNET_STOP_PG	0x80	/* Last page +1 of RX ring */
@@ -68,7 +70,7 @@
 #define SOCKET_START_PG	0x01
 #define SOCKET_STOP_PG	0xff
 
-#define PCNET_RDC_TIMEOUT 0x02	/* Max wait in jiffies for Tx RDC */
+#define PCNET_RDC_TIMEOUT (2*HZ/100)	/* Max wait in jiffies for Tx RDC */
 
 static char *if_names[] = { "auto", "10baseT", "10base2"};
 
@@ -77,7 +79,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"pcnet_cs.c 1.144 2001/11/07 04:06:56 (David Hinds)";
+"pcnet_cs.c 1.149 2002/06/29 06:27:37 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -119,6 +121,7 @@ static int pcnet_event(event_t event, int priority,
 static int pcnet_open(struct net_device *dev);
 static int pcnet_close(struct net_device *dev);
 static int ei_ioctl(struct net_device *dev, struct ifreq *rq, int cmd);
+static int do_ioctl_light(struct net_device *dev, struct ifreq *rq, int cmd);
 static void ei_irq_wrapper(int irq, void *dev_id, struct pt_regs *regs);
 static void ei_watchdog(u_long arg);
 static void pcnet_reset_8390(struct net_device *dev);
@@ -414,15 +417,16 @@ static hw_info_t *get_hwinfo(dev_link_t *link)
 	return NULL;
     }
 
-    virt = ioremap(req.Base, req.Size);
+    virt = ioremap(req.Base, req.Size << PCMCIA_ADDRESS_SHIFT);
     mem.Page = 0;
     for (i = 0; i < NR_INFO; i++) {
 	mem.CardOffset = hw_info[i].offset & ~(req.Size-1);
 	CardServices(MapMemPage, link->win, &mem);
-	base = &virt[hw_info[i].offset & (req.Size-1)];
-	if ((readb(base+0) == hw_info[i].a0) &&
-	    (readb(base+2) == hw_info[i].a1) &&
-	    (readb(base+4) == hw_info[i].a2))
+	base = &virt[(hw_info[i].offset << PCMCIA_ADDRESS_SHIFT) &
+		     ((req.Size << PCMCIA_ADDRESS_SHIFT)-1)];
+	if ((readb(base+(0<<PCMCIA_ADDRESS_SHIFT)) == hw_info[i].a0) &&
+	    (readb(base+(2<<PCMCIA_ADDRESS_SHIFT)) == hw_info[i].a1) &&
+	    (readb(base+(4<<PCMCIA_ADDRESS_SHIFT)) == hw_info[i].a2))
 	    break;
     }
     if (i < NR_INFO) {
@@ -720,6 +724,9 @@ static void pcnet_config(dev_link_t *link)
 	goto failed;
     }
 
+#ifdef CONFIG_DBMX1_TPM102
+    tpm102_cs5_8bit();
+#endif
     hw_info = get_hwinfo(link);
     if (hw_info == NULL)
 	hw_info = get_prom(link);
@@ -769,18 +776,21 @@ static void pcnet_config(dev_link_t *link)
 
     strcpy(info->node.dev_name, dev->name);
     link->dev = &info->node;
-    link->state &= ~DEV_CONFIG_PENDING;
 
     if (info->flags & (IS_DL10019|IS_DL10022)) {
 	u_char id = inb(dev->base_addr + 0x1a);
 	dev->do_ioctl = &ei_ioctl;
 	mii_phy_probe(dev);
+	if ((id == 0x30) && !info->pna_phy && (info->eth_phy == 4))
+	    info->eth_phy = 0;
 	printk(KERN_INFO "%s: NE2000 (DL100%d rev %02x): ",
 	       dev->name, ((info->flags & IS_DL10022) ? 22 : 19), id);
 	if (info->pna_phy)
 	    printk("PNA, ");
-    } else
+    } else {
 	printk(KERN_INFO "%s: NE2000 Compatible: ", dev->name);
+ 	dev->do_ioctl = &do_ioctl_light;	
+    }
     printk("io %#3lx, irq %d,", dev->base_addr, dev->irq);
     if (info->flags & USE_SHMEM)
 	printk (" mem %#5lx,", dev->mem_start);
@@ -789,12 +799,14 @@ static void pcnet_config(dev_link_t *link)
     printk(" hw_addr ");
     for (i = 0; i < 6; i++)
 	printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : "\n"));
+    link->state &= ~DEV_CONFIG_PENDING;
     return;
 
 cs_failed:
     cs_error(link->handle, last_fn, last_ret);
 failed:
     pcnet_release((u_long)link);
+    link->state &= ~DEV_CONFIG_PENDING;
     return;
 } /* pcnet_config */
 
@@ -858,7 +870,7 @@ static int pcnet_event(event_t event, int priority,
 	}
 	break;
     case CS_EVENT_CARD_INSERTION:
-	link->state |= DEV_PRESENT;
+	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
 	pcnet_config(link);
 	break;
     case CS_EVENT_PM_SUSPEND:
@@ -897,8 +909,8 @@ static int pcnet_event(event_t event, int priority,
 
 ======================================================================*/
 
-#define DLINK_GPIO		0x1c
-#define DLINK_DIAG		0x1d
+#define DLINK_GPIO		(0x1c<<PCMCIA_ADDRESS_SHIFT)
+#define DLINK_DIAG		(0x1d<<PCMCIA_ADDRESS_SHIFT)
 #define MDIO_SHIFT_CLK		0x80
 #define MDIO_DATA_OUT		0x40
 #define MDIO_DIR_WRITE		0x30
@@ -1053,6 +1065,7 @@ static int pcnet_close(struct net_device *dev)
 
     DEBUG(2, "pcnet_close('%s')\n", dev->name);
 
+    ei_close(dev);
     free_irq(dev->irq, dev);
     
     link->open--;
@@ -1190,7 +1203,7 @@ static void ei_watchdog(u_long arg)
 	}
 	info->link_status = link;
     }
-    if (info->pna_phy && (jiffies - info->mii_reset > 6*HZ)) {
+    if (info->pna_phy && time_after(jiffies, info->mii_reset + 6*HZ)) {
 	link = mdio_read(mii_addr, info->eth_phy, 1) & 0x0004;
 	if (((info->phy_id == info->pna_phy) && link) ||
 	    ((info->phy_id != info->pna_phy) && !link)) {
@@ -1213,12 +1226,37 @@ reschedule:
 
 /*====================================================================*/
 
+static int netdev_ethtool_ioctl(struct net_device *dev, void *useraddr)
+{
+	u32 ethcmd;
+	
+	if (copy_from_user(&ethcmd, useraddr, sizeof(ethcmd)))
+		return -EFAULT;
+	
+	switch (ethcmd) {
+	case ETHTOOL_GDRVINFO: {
+		struct ethtool_drvinfo info = {ETHTOOL_GDRVINFO};
+		strncpy(info.driver, "pcnet_cs", sizeof(info.driver)-1);
+		if (copy_to_user(useraddr, &info, sizeof(info)))
+			return -EFAULT;
+		return 0;
+	}
+	}
+	
+	return -EOPNOTSUPP;
+}
+
+/*====================================================================*/
+
+
 static int ei_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 {
     pcnet_dev_t *info = (pcnet_dev_t *)dev;
     u16 *data = (u16 *)&rq->ifr_data;
     ioaddr_t mii_addr = dev->base_addr + DLINK_GPIO;
     switch (cmd) {
+    case SIOCETHTOOL:
+        return netdev_ethtool_ioctl(dev, (void *) rq->ifr_data);
     case SIOCDEVPRIVATE:
 	data[0] = info->phy_id;
     case SIOCDEVPRIVATE+1:
@@ -1235,6 +1273,17 @@ static int ei_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 
 /*====================================================================*/
 
+static int do_ioctl_light(struct net_device *dev, struct ifreq *rq, int cmd)
+{
+    switch (cmd) {
+        case SIOCETHTOOL:
+            return netdev_ethtool_ioctl(dev, (void *) rq->ifr_data);
+    }	    
+    return -EOPNOTSUPP;    
+}
+
+/*====================================================================*/
+
 static void dma_get_8390_hdr(struct net_device *dev,
 			     struct e8390_pkt_hdr *hdr,
 			     int ring_page)
@@ -1256,8 +1305,14 @@ static void dma_get_8390_hdr(struct net_device *dev,
     outb_p(ring_page, nic_base + EN0_RSARHI);
     outb_p(E8390_RREAD+E8390_START, nic_base + PCNET_CMD);
 
+#ifdef CONFIG_DBMX1_TPM102
+    tpm102_cs5_16bit();
+#endif
     insw(nic_base + PCNET_DATAPORT, hdr,
 	    sizeof(struct e8390_pkt_hdr)>>1);
+#ifdef CONFIG_DBMX1_TPM102
+    tpm102_cs5_8bit();
+#endif
     /* Fix for big endian systems */
     hdr->count = le16_to_cpu(hdr->count);
 
@@ -1292,7 +1347,13 @@ static void dma_block_input(struct net_device *dev, int count,
     outb_p(ring_offset >> 8, nic_base + EN0_RSARHI);
     outb_p(E8390_RREAD+E8390_START, nic_base + PCNET_CMD);
 
+#ifdef CONFIG_DBMX1_TPM102
+    tpm102_cs5_16bit();
+#endif
     insw(nic_base + PCNET_DATAPORT,buf,count>>1);
+#ifdef CONFIG_DBMX1_TPM102
+    tpm102_cs5_8bit();
+#endif
     if (count & 0x01)
 	buf[count-1] = inb(nic_base + PCNET_DATAPORT), xfer_count++;
 
@@ -1365,7 +1426,13 @@ static void dma_block_output(struct net_device *dev, int count,
     outb_p(start_page, nic_base + EN0_RSARHI);
 
     outb_p(E8390_RWRITE+E8390_START, nic_base + PCNET_CMD);
+#ifdef CONFIG_DBMX1_TPM102
+    tpm102_cs5_16bit();
+#endif
     outsw(nic_base + PCNET_DATAPORT, buf, count>>1);
+#ifdef CONFIG_DBMX1_TPM102
+    tpm102_cs5_8bit();
+#endif
 
     dma_start = jiffies;
 
@@ -1392,7 +1459,7 @@ static void dma_block_output(struct net_device *dev, int count,
 #endif
 
     while ((inb_p(nic_base + EN0_ISR) & ENISR_RDC) == 0)
-	if (jiffies - dma_start > PCNET_RDC_TIMEOUT) {
+	if (time_after(jiffies, dma_start + PCNET_RDC_TIMEOUT)) {
 	    printk(KERN_NOTICE "%s: timeout waiting for Tx RDC.\n",
 		   dev->name);
 	    pcnet_reset_8390(dev);
@@ -1586,7 +1653,7 @@ static int __init init_pcnet_cs(void)
     if (serv.Revision != CS_RELEASE_CODE) {
 	printk(KERN_NOTICE "pcnet_cs: Card Services release "
 	       "does not match!\n");
-	return -1;
+	return -EINVAL;
     }
     register_pccard_driver(&dev_info, &pcnet_attach, &pcnet_detach);
     return 0;
