@@ -33,6 +33,7 @@
 	12-Nov-2001 Lineo Japan, Inc.
 	30-Jul-2002 Lineo Japan, Inc.  for 2.4.18
 	12-Dec-2002 Sharp Corporation for Poodle and Corgi
+	01-Jun-2004 Lineo Solutions, Inc.  for PXA27x
 ======================================================================*/
 
 #define __NO_VERSION__
@@ -99,8 +100,13 @@ static int force_8bit_access_check_and_set(
 	char sample1[FORCE_8BIT_ACCESS_CHECK_LEN];
 	char sample2[FORCE_8BIT_ACCESS_CHECK_LEN];
 	char *p = sample1;
+#if defined(CONFIG_CPU_PXA27X)
+	GPSR(GPIO54_nPCE_2) = GPSR(GPIO54_nPCE_2);
+	set_GPIO_mode(GPIO54_nPCE_2|GPIO_OUT);
+#else
 	GPSR(GPIO53_nPCE_2) = GPSR(GPIO53_nPCE_2);
 	set_GPIO_mode(GPIO53_nPCE_2|GPIO_OUT);
+#endif
 	for(;;){
 		int i, j;
 		for(i=j=0; i<FORCE_8BIT_ACCESS_CHECK_LEN; i++, j+= inc)
@@ -108,12 +114,20 @@ static int force_8bit_access_check_and_set(
 		if( p == sample2 )
 			break;
 		p = sample2;
+#if defined(CONFIG_CPU_PXA27X)
+		set_GPIO_mode(GPIO54_nPCE_2_MD);
+#else
 		set_GPIO_mode(GPIO53_nPCE_2_MD);
+#endif
 	};
 	if( !memcmp(sample1, sample2, FORCE_8BIT_ACCESS_CHECK_LEN) )
 		return  1;
 	/* force 8bit access */
+#if defined(CONFIG_CPU_PXA27X)
+	set_GPIO_mode(GPIO54_nPCE_2|GPIO_OUT);
+#else
 	set_GPIO_mode(GPIO53_nPCE_2|GPIO_OUT);
+#endif
 	return 2;
 }
 #endif
@@ -138,7 +152,7 @@ static void set_cis_map(socket_info_t *s, pccard_mem_map *mem)
 	if (s->cis_virt)
 	    bus_iounmap(s->cap.bus, s->cis_virt);
 	s->cis_virt = bus_ioremap(s->cap.bus, mem->sys_start,
-				  s->cap.map_size);
+				  s->cap.map_size << PCMCIA_ADDRESS_SHIFT);
     }
 }
 
@@ -186,6 +200,7 @@ void read_cis_mem(socket_info_t *s, int attr, u_int addr,
     } else {
 	u_int inc = 1;
 	if (attr) { mem->flags |= MAP_ATTRIB; inc++; addr *= 2; }
+	inc <<= PCMCIA_ADDRESS_SHIFT;
 	sys += (addr & (s->cap.map_size-1));
 	mem->card_start = addr & ~(s->cap.map_size-1);
 	while (len) {
@@ -196,9 +211,11 @@ void read_cis_mem(socket_info_t *s, int attr, u_int addr,
 		    force_8bit_access_check_and_set(
 			s->cap.bus, s->cis_virt, inc);
 #endif
-	    sys = s->cis_virt + (addr & (s->cap.map_size-1));
+	    sys = s->cis_virt +
+		((addr & (s->cap.map_size-1)) << PCMCIA_ADDRESS_SHIFT);
 	    for ( ; len > 0; len--, buf++, sys += inc) {
-		if (sys == s->cis_virt+s->cap.map_size) break;
+		if (sys == s->cis_virt+(s->cap.map_size<<PCMCIA_ADDRESS_SHIFT))
+		    break;
 #if defined(CONFIG_SABINAL_DISCOVERY)
 		for(; busy_wait_max_time > 0; busy_wait_max_time--){
 		    int val;
@@ -247,12 +264,15 @@ void write_cis_mem(socket_info_t *s, int attr, u_int addr,
     } else {
 	int inc = 1;
 	if (attr & IS_ATTR) { mem->flags |= MAP_ATTRIB; inc++; addr *= 2; }
+	inc <<= PCMCIA_ADDRESS_SHIFT;
 	mem->card_start = addr & ~(s->cap.map_size-1);
 	while (len) {
 	    set_cis_map(s, mem);
-	    sys = s->cis_virt + (addr & (s->cap.map_size-1));
+	    sys = s->cis_virt +
+		((addr & (s->cap.map_size-1)) << PCMCIA_ADDRESS_SHIFT);
 	    for ( ; len > 0; len--, buf++, sys += inc) {
-		if (sys == s->cis_virt+s->cap.map_size) break;
+		if (sys == s->cis_virt+(s->cap.map_size<<PCMCIA_ADDRESS_SHIFT))
+		    break;
 		bus_writeb(s->cap.bus, *buf, sys);
 	    }
 	    mem->card_start += s->cap.map_size;
@@ -328,11 +348,13 @@ static int setup_cis_mem(socket_info_t *s)
 	validate_mem(cis_readable, checksum_match, low, s);
 	s->cis_mem.sys_start = 0;
 	vs = NULL;
+#ifndef CONFIG_TOADKK_TCS8000
 	if (find_mem_region(&s->cis_mem.sys_start, s->cap.map_size,
 			    s->cap.map_size, low, "card services", s)) {
 	    printk(KERN_NOTICE "cs: unable to map card memory!\n");
 	    return CS_OUT_OF_RESOURCE;
 	}
+#endif
 	s->cis_mem.sys_stop = s->cis_mem.sys_start+s->cap.map_size-1;
 	s->cis_virt = bus_ioremap(s->cap.bus, s->cis_mem.sys_start,
 				  s->cap.map_size);
@@ -1147,6 +1169,12 @@ static int parse_cftable_entry(tuple_t *tuple,
     if (features & 0x08) {
 	p = parse_io(p, q, &entry->io);
 	if (p == NULL) return CS_BAD_TUPLE;
+#ifdef CONFIG_DBMX1_TPM102
+	if (entry->io.flags & CISTPL_IO_16BIT)
+	    tpm102_cs5_16bit();
+	else
+	    tpm102_cs5_8bit();
+#endif
     } else
 	entry->io.nwin = 0;
     
