@@ -15,103 +15,8 @@
  *
  *	linux-arm-kernel@lists.arm.linux.org.uk
  *
- * Clean patches should be sent to the ARM Linux Patch System.  Please see the
- * following web page for more information:
- *
- *	http://www.arm.linux.org.uk/developer/patches/info.shtml
- *
- * Thank you.
- *
  *
  * Code Status:
- * 1999/04/01:
- *	- Driver appears to be working for Brutus 320x200x8bpp mode.  Other
- *	  resolutions are working, but only the 8bpp mode is supported.
- *	  Changes need to be made to the palette encode and decode routines
- *	  to support 4 and 16 bpp modes.  
- *	  Driver is not designed to be a module.  The FrameBuffer is statically
- *	  allocated since dynamic allocation of a 300k buffer cannot be 
- *	  guaranteed. 
- *
- * 1999/06/17:
- *	- FrameBuffer memory is now allocated at run-time when the
- *	  driver is initialized.    
- *
- * 2000/04/10: Nicolas Pitre <nico@cam.org>
- *	- Big cleanup for dynamic selection of machine type at run time.
- *
- * 2000/07/19: Jamey Hicks <jamey@crl.dec.com>
- *	- Support for Bitsy aka Compaq iPAQ H3600 added.
- *
- * 2000/08/07: Tak-Shing Chan <tchan.rd@idthk.com>
- *	       Jeff Sutherland <jsutherland@accelent.com>
- *	- Resolved an issue caused by a change made to the Assabet's PLD 
- *	  earlier this year which broke the framebuffer driver for newer 
- *	  Phase 4 Assabets.  Some other parameters were changed to optimize
- *	  for the Sharp display.
- *
- * 2000/08/09: Cliff Brake <cbrake@accelent.com>
- *      - modified for Accelent
- * 
- * 2000/08/09: Kunihiko IMAI <imai@vasara.co.jp>
- *	- XP860 support added
- *
- * 2000/08/19: Mark Huang <mhuang@livetoy.com>
- *	- Allows standard options to be passed on the kernel command line
- *	  for most common passive displays.
- *
- * 2000/08/29:
- *	- s/save_flags_cli/local_irq_save/
- *	- remove unneeded extra save_flags_cli in sa1100fb_enable_lcd_controller
- *
- * 2000/10/10: Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
- *	- Updated LART stuff. Fixed some minor bugs.
- *
- * 2000/10/30: Murphy Chen <murphy@mail.dialogue.com.tw>
- *	- Pangolin support added
- *
- * 2000/10/31: Roman Jordan <jor@hoeft-wessel.de>
- *	- Huw Webpanel support added
- *
- * 2000/11/23: Eric Peng <ericpeng@coventive.com>
- *	- Freebird add
- *
- * 2001/02/07: Jamey Hicks <jamey.hicks@compaq.com> 
- *	       Cliff Brake <cbrake@accelent.com>
- *	- Added PM callback
- *
- * 2001/05/26: <rmk@arm.linux.org.uk>
- *	- Fix 16bpp so that (a) we use the right colours rather than some
- *	  totally random colour depending on what was in page 0, and (b)
- *	  we don't de-reference a NULL pointer.
- *	- remove duplicated implementation of consistent_alloc()
- *	- convert dma address types to dma_addr_t
- *	- remove unused 'montype' stuff
- *	- remove redundant zero inits of init_var after the initial
- *	  memzero.
- *	- remove allow_modeset (acornfb idea does not belong here)
- *
- * 2001/05/28: <rmk@arm.linux.org.uk>
- *	- massive cleanup - move machine dependent data into structures
- *	- I've left various #warnings in - if you see one, and know
- *	  the hardware concerned, please get in contact with me.
- *
- * 2001/05/31: <rmk@arm.linux.org.uk>
- *	- Fix LCCR1 HSW value, fix all machine type specifications to
- *	  keep values in line.  (Please check your machine type specs)
- *
- * 2001/06/10: <rmk@arm.linux.org.uk>
- *	- Fiddle with the LCD controller from task context only; mainly
- *	  so that we can run with interrupts on, and sleep.
- *	- Convert #warnings into #errors.  No pain, no gain. ;)
- *
- * 2001/06/14: <rmk@arm.linux.org.uk>
- *	- Make the palette BPS value for 12bpp come out correctly.
- *	- Take notice of "greyscale" on any colour depth.
- *	- Make truecolor visuals use the RGB channel encoding information.
- *
- * 2001/07/02: <rmk@arm.linux.org.uk>
- *	- Fix colourmap problems.
  *
  * 2001/08/03: <cbrake@accelent.com>
  *      - Ported from SA1100 to PXA250
@@ -129,6 +34,7 @@
 #include <linux/delay.h>
 #include <linux/pm.h>
 #include <linux/init.h>
+#include <linux/notifier.h>
 #include <linux/cpufreq.h>
 
 #include <asm/hardware.h>
@@ -298,11 +204,12 @@ pxafb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
 
 	switch (fbi->fb.disp->visual) {
 	case FB_VISUAL_TRUECOLOR:
+	case FB_VISUAL_DIRECTCOLOR:
 		/*
 		 * 12 or 16-bit True Colour.  We encode the RGB value
 		 * according to the RGB bitfield information.
 		 */
-		if (regno < 16) {
+		if (regno <= 16) {
 			u16 *pal = fbi->fb.pseudo_palette;
 
 			val  = chan_to_field(red, &fbi->fb.var.red);
@@ -365,8 +272,12 @@ static int pxafb_validate_var(struct fb_var_screeninfo *var,
 		break;
 
 	case 16:
-		/* cerf pda 16 bits works fine in passive mode, so don't complain */
-		if (machine_is_pxa_cerf()) {
+		/* 
+		 * 16 bits works apparemtly fine in passive mode for those,
+		 * so don't complain
+		 */
+		if (machine_is_lubbock() ||
+		    machine_is_pxa_cerf()) {
 			ret = 0;
 		} else
 			/* make sure we are in active mode */
@@ -384,23 +295,6 @@ static int pxafb_validate_var(struct fb_var_screeninfo *var,
 static inline void pxafb_set_truecolor(u_int is_true_color)
 {
 	DPRINTK("true_color = %d\n", is_true_color);
-#ifdef CONFIG_SA1100_ASSABET
-	if (machine_is_assabet()) {
-#if 1
-		// phase 4 or newer Assabet's
-		if (is_true_color)
-			BCR_set(BCR_LCD_12RGB);
-		else
-			BCR_clear(BCR_LCD_12RGB);
-#else
-		// older Assabet's
-		if (is_true_color)
-			BCR_clear(BCR_LCD_12RGB);
-		else
-			BCR_set(BCR_LCD_12RGB);
-#endif
-	}
-#endif
 }
 
 static void
@@ -816,6 +710,7 @@ static int pxafb_activate_var(struct fb_var_screeninfo *var, struct pxafb_info *
 			fbi->fb.fix.id, var->lower_margin);
 #endif
 
+#if defined (CONFIG_PXA_CERF_PDA)
 	new_regs.lccr0 = fbi->lccr0;
 	new_regs.lccr1 =
 		LCCR1_DisWdth(var->xres) +
@@ -833,6 +728,44 @@ static int pxafb_activate_var(struct fb_var_screeninfo *var, struct pxafb_info *
 		|
 		(var->sync & FB_SYNC_HOR_HIGH_ACT ? LCCR3_HorSnchH : LCCR3_HorSnchL) |
 		(var->sync & FB_SYNC_VERT_HIGH_ACT ? LCCR3_VrtSnchH : LCCR3_VrtSnchL);
+#elif defined (CONFIG_FB_PXA_QVGA)
+	new_regs.lccr0 = fbi->lccr0;
+	new_regs.lccr1 =
+		LCCR1_DisWdth(var->xres) +
+		LCCR1_HorSnchWdth(var->hsync_len) +
+		LCCR1_BegLnDel(var->left_margin) +
+		LCCR1_EndLnDel(var->right_margin);
+	new_regs.lccr2 =
+		LCCR2_DisHght(var->yres) +
+		LCCR2_VrtSnchWdth(var->vsync_len) +
+		LCCR2_BegFrmDel(var->upper_margin) +
+		LCCR2_EndFrmDel(var->lower_margin);
+	new_regs.lccr3 = fbi->lccr3;
+#else
+	// FIXME using hardcoded values for now
+	new_regs.lccr0 = fbi->lccr0;
+//		|
+//		LCCR0_LEN | LCCR0_LDM | LCCR0_BAM |
+//		LCCR0_ERM | LCCR0_LtlEnd | LCCR0_DMADel(0);
+
+	new_regs.lccr1 = 0x3030A7F;
+//		LCCR1_DisWdth(var->xres) +
+//		LCCR1_HorSnchWdth(var->hsync_len) +
+//		LCCR1_BegLnDel(var->left_margin) +
+//		LCCR1_EndLnDel(var->right_margin);
+
+	new_regs.lccr2 = 0x4EF;
+//		LCCR2_DisHght(var->yres) +
+//		LCCR2_VrtSnchWdth(var->vsync_len) +
+//		LCCR2_BegFrmDel(var->upper_margin) +
+//		LCCR2_EndFrmDel(var->lower_margin);
+
+	new_regs.lccr3 = fbi->lccr3;
+//	|
+//		(var->sync & FB_SYNC_HOR_HIGH_ACT ? LCCR3_HorSnchH : LCCR3_HorSnchL) |
+//		(var->sync & FB_SYNC_VERT_HIGH_ACT ? LCCR3_VrtSnchH : LCCR3_VrtSnchL) |
+//		LCCR3_ACBsCntOff;
+#endif
 
 //	if (pcd)
 //		new_regs.lccr3 |= LCCR3_PixClkDiv(pcd);
@@ -1469,45 +1402,10 @@ failed:
 	return ret;
 }
 
-int __init pxafb_setup(char *options)
-{
-#if 0
-	char *this_opt;
 
-	if (!options || !*options)
-		return 0;
-
-	for (this_opt = strtok(options, ","); this_opt;
-	     this_opt = strtok(NULL, ",")) {
-
-		if (!strncmp(this_opt, "bpp:", 4))
-			current_par.max_bpp =
-			    simple_strtoul(this_opt + 4, NULL, 0);
-
-		if (!strncmp(this_opt, "lccr0:", 6))
-			lcd_shadow.lccr0 =
-			    simple_strtoul(this_opt + 6, NULL, 0);
-		if (!strncmp(this_opt, "lccr1:", 6)) {
-			lcd_shadow.lccr1 =
-			    simple_strtoul(this_opt + 6, NULL, 0);
-			current_par.max_xres =
-			    (lcd_shadow.lccr1 & 0x3ff) + 16;
-		}
-		if (!strncmp(this_opt, "lccr2:", 6)) {
-			lcd_shadow.lccr2 =
-			    simple_strtoul(this_opt + 6, NULL, 0);
-			current_par.max_yres =
-			    (lcd_shadow.
-			     lccr0 & LCCR0_SDS) ? ((lcd_shadow.
-						    lccr2 & 0x3ff) +
-						   1) *
-			    2 : ((lcd_shadow.lccr2 & 0x3ff) + 1);
-		}
-		if (!strncmp(this_opt, "lccr3:", 6))
-			lcd_shadow.lccr3 =
-			    simple_strtoul(this_opt + 6, NULL, 0);
-	}
+#ifdef MODULE
+module_init(pxafb_init);
 #endif
-	return 0;
 
-}
+MODULE_DESCRIPTION("loadable framebuffer driver for PXA");
+MODULE_LICENSE("GPL");
