@@ -51,6 +51,8 @@
  *			 :  proc_misc.c. The rest may eventually go into
  *			 :  base.c too.
  *
+ * David McCullough  :  added NO_MM support <davidm@lineo.com>
+ *
  * ChangeLog:
  *     23-Nov-2002 SHARP  add pmem
  *     04-Apr-2003 Sharp for ARM FCSE
@@ -193,6 +195,7 @@ static inline char * task_state(struct task_struct *p, char *buffer)
 
 static inline char * task_mem(struct mm_struct *mm, char *buffer)
 {
+#ifndef NO_MM
 	struct vm_area_struct * vma;
 	unsigned long data = 0, stack = 0;
 	unsigned long exec = 0, lib = 0;
@@ -229,6 +232,54 @@ static inline char * task_mem(struct mm_struct *mm, char *buffer)
 		data - stack, stack,
 		exec - lib, lib);
 	up_read(&mm->mmap_sem);
+#else
+	unsigned long bytes = 0, sbytes = 0, slack = 0;
+	struct mm_tblock_struct * tblock;
+        
+	/* Logic: we've got two memory sums for each process, "shared", and
+	 * "non-shared". Shared memory may get counted more then once, for
+	 * each process that owns it. Non-shared memory is counted
+	 * accurately.
+	 *
+	 *	-- Kenneth Albanowski
+	 */
+
+	down_read(&mm->mmap_sem);
+	for (tblock = &mm->tblock; tblock; tblock = tblock->next) {
+		if (tblock->rblock) {
+			bytes += ksize(tblock);
+			if (atomic_read(&mm->mm_count) > 1 ||
+					tblock->rblock->refcount > 1) {
+				sbytes += ksize(tblock->rblock->kblock);
+				sbytes += ksize(tblock->rblock) ;
+			} else {
+				bytes += ksize(tblock->rblock->kblock);
+				bytes += ksize(tblock->rblock) ;
+				slack += ksize(tblock->rblock->kblock) - tblock->rblock->size;
+			}
+		}
+	}
+	
+	((atomic_read(&mm->mm_count) > 1) ? sbytes : bytes)
+			+= ksize(mm);
+	(current->fs && atomic_read(&current->fs->count) > 1 ? sbytes : bytes)
+			+= ksize(current->fs);
+	(current->files && atomic_read(&current->files->count) > 1 ? sbytes : bytes)
+			+= ksize(current->files);
+	(current->sig && atomic_read(&current->sig->count) > 1 ? sbytes : bytes)
+			+= ksize(current->sig);
+	bytes += ksize(current); /* includes kernel stack */
+
+	buffer += sprintf(buffer,
+		"Mem:\t%8lu bytes\n"
+		"Slack:\t%8lu bytes\n"
+		"Shared:\t%8lu bytes\n",
+		bytes,
+		slack,
+		sbytes);
+
+	up_read(&mm->mmap_sem);
+#endif
 	return buffer;
 }
 
@@ -241,6 +292,8 @@ static void collect_sigign_sigcatch(struct task_struct *p, sigset_t *ign,
 	sigemptyset(ign);
 	sigemptyset(catch);
 
+	spin_lock_irq(&p->sigmask_lock);
+
 	if (p->sig) {
 		k = p->sig->action;
 		for (i = 1; i <= _NSIG; ++i, ++k) {
@@ -250,6 +303,7 @@ static void collect_sigign_sigcatch(struct task_struct *p, sigset_t *ign,
 				sigaddset(catch, i);
 		}
 	}
+	spin_unlock_irq(&p->sigmask_lock);
 }
 
 static inline char * task_sig(struct task_struct *p, char *buffer)
@@ -332,13 +386,28 @@ int proc_pid_stat(struct task_struct *task, char * buffer)
 	}
 	task_unlock(task);
 	if (mm) {
+#ifndef NO_MM
 		struct vm_area_struct *vma;
+#endif
 		down_read(&mm->mmap_sem);
+#ifndef NO_MM
 		vma = mm->mmap;
 		while (vma) {
 			vsize += vma->vm_end - vma->vm_start;
 			vma = vma->vm_next;
 		}
+#else /* !NO_MM */
+		vsize = 0;
+		{
+			struct mm_tblock_struct *tbp = &mm->tblock;
+
+			while (tbp) {
+				if (tbp->rblock)
+					vsize += ksize(tbp->rblock->kblock);
+				tbp = tbp->next;
+			}
+		}
+#endif /* !NO_MM */
 		eip = KSTK_EIP(task);
 		esp = KSTK_ESP(task);
 		up_read(&mm->mmap_sem);
@@ -408,6 +477,8 @@ int proc_pid_stat(struct task_struct *task, char * buffer)
 	return res;
 }
 		
+#ifndef NO_MM
+
 static inline void statm_pte_range(pmd_t * pmd, unsigned long address, unsigned long size,
 	int * pages, int * shared, int * dirty, int * total)
 {
@@ -483,6 +554,8 @@ static void statm_pgd_range(pgd_t * pgd, unsigned long address, unsigned long en
 	}
 }
 
+#endif /* !NO_MM */
+
 int proc_pid_statm(struct task_struct *task, char * buffer)
 {
 	struct mm_struct *mm;
@@ -494,8 +567,11 @@ int proc_pid_statm(struct task_struct *task, char * buffer)
 		atomic_inc(&mm->mm_users);
 	task_unlock(task);
 	if (mm) {
+#ifndef NO_MM
 		struct vm_area_struct * vma;
+#endif
 		down_read(&mm->mmap_sem);
+#ifndef NO_MM
 		vma = mm->mmap;
 		while (vma) {
 			pgd_t *pgd = pgd_offset(mm, vma->vm_start);
@@ -516,6 +592,39 @@ int proc_pid_statm(struct task_struct *task, char * buffer)
 				drs += pages;
 			vma = vma->vm_next;
 		}
+#else /* !NO_MM */
+		/* DAVIDM - may be able to clean this up a bit */
+		{
+			struct mm_tblock_struct *tbp = &mm->tblock;
+
+			size += ksize(mm);
+			while (tbp) {
+				if (tbp->next)
+					size += ksize(tbp->next);
+				if (tbp->rblock) {
+					size += ksize(tbp->rblock);
+					size += ksize(tbp->rblock->kblock);
+					if (atomic_read(&mm->mm_count) > 1 ||
+							(tbp->rblock->refcount > 1))
+						share += tbp->rblock->size;
+				}
+				tbp = tbp->next;
+			}
+
+			size += (trs = mm->end_code - mm->start_code);
+			size += (drs = mm->start_stack - mm->start_data);
+			dt  = 0;
+			lrs = 0;
+			resident = size;
+
+			/* User programs expect the units to be pages. */
+			size = (size + PAGE_SIZE - 1) >> PAGE_SHIFT;
+			resident = (resident + PAGE_SIZE - 1) >> PAGE_SHIFT;
+			share = (share + PAGE_SIZE - 1) >> PAGE_SHIFT;
+			trs = (trs + PAGE_SIZE - 1) >> PAGE_SHIFT;
+			drs = (drs + PAGE_SIZE - 1) >> PAGE_SHIFT;
+		}
+#endif /* !NO_MM */
 		up_read(&mm->mmap_sem);
 		mmput(mm);
 	}
@@ -523,6 +632,7 @@ int proc_pid_statm(struct task_struct *task, char * buffer)
 		       size, resident, share, trs, lrs, drs, dt);
 }
 
+#ifndef NO_MM
 /*
  * The way we support synthetic files > 4K
  * - without storing their contents in some buffer and
@@ -601,10 +711,12 @@ static int proc_pid_maps_get_line (char *buf, struct vm_area_struct *map)
 		line[len++] = '\n';
 	return len;
 }
+#endif /* NO_MM */
 
 ssize_t proc_pid_read_maps (struct task_struct *task, struct file * file, char * buf,
 			  size_t count, loff_t *ppos)
 {
+#ifndef NO_MM
 	struct mm_struct *mm;
 	struct vm_area_struct * map;
 	char *tmp, *kbuf;
@@ -688,6 +800,10 @@ out_free1:
 	free_page((unsigned long)kbuf);
 out:
 	return retval;
+#else /* !NO_MM */
+	/* DAVIDM - no idea if this one is possible :-) */
+	return(0);
+#endif /* !NO_MM */
 }
 
 #ifdef CONFIG_SMP
